from cassandra.cluster import Cluster
from cassandra.auth import PlainTextAuthProvider
from .Config import Config
import sys

class CyaniteCassandra():
    """
    Connect to Cyanite Cassandra
    """

    def __init__(self, config):
        self.config = config
        auth_provider = None
        if config.clusteruser():
            auth_provider = PlainTextAuthProvider(
                    username=config.clusteruser(),
                    password=config.clusterpass()
                    )
        self.cluster = Cluster(
                config.cluster(),
                auth_provider=auth_provider
                )
        self.session = self.cluster.connect(config.keyspace())
        self.deletequery = self.session.prepare(
            """
            DELETE FROM metric
            WHERE tenant=''
                AND period=?
                AND rollup=?
                AND path=?
            """)

    def delete(self, metric):
        rollups = self.config.rollups()
        if len(rollups) == 0:
            sys.stderr.write("cassandra cannot delete without rollups defined in config\n")
            sys.exit(1)
        for (rollup, interval) in rollups:
            sys.stdout.write("metric delete %s rollup %d interval %d\n" % (
                metric, rollup, interval))
            sys.stdout.flush()
            rows = self.session.execute(self.deletequery,
                    (rollup, interval, metric))



import argparse
import fileinput

from .Config import Config
from .CyaniteCassandra import CyaniteCassandra
from .CyaniteMetrics import CyaniteMetrics
from .CyanitePaths import CyanitePaths
from .util import common_parser, catch_sigint

def cyanite_list():
    catch_sigint()
    parser = common_parser('List metrics')

    parser.add_argument(
        'metric',
        metavar='METRIC',
        nargs='*',
        type=str,
        help='metric name to search for')

    args = parser.parse_args()
    config = Config(args.config_file)
    if args.verbose:
        config.setverbose()

    paths = CyanitePaths(config)

    if len(args.metric) == 0:
        paths.printpaths('*')
    else:
        for metric in args.metric:
            paths.printpaths(metric)

def cyanite_delete():
    catch_sigint()
    parser = common_parser('Delete metrics')

    parser.add_argument(
        'metric',
        metavar='METRIC',
        nargs='*',
        type=str,
        help='metric name to delete')

    args = parser.parse_args()
    config = Config(args.config_file)
    if args.verbose:
        config.setverbose()

    cyanite = CyaniteCassandra(config)
    paths = CyanitePaths(config)

    for metric in args.metric:
        cyanite.delete(metric)
        if config.espathindex():
            paths.delete(metric)

    if len(args.metric) == 0:
        for metric in fileinput.input(args.file):
            cyanite.delete(metric.rstrip('\n'))
            if config.espathindex():
                paths.delete(metric.rstrip('\n'))

def cyanite_prune():
    catch_sigint()
    parser = common_parser('Prune metrics')

    parser.add_argument(
        '-s', '--seconds',
        default='101520',
        help='seconds of inactivity to prune')

    parser.add_argument(
        'metric',
        metavar='METRIC',
        nargs='*',
        type=str,
        help='metric name to prune')

    args = parser.parse_args()
    config = Config(args.config_file)
    if args.verbose:
        config.setverbose()
    if args.seconds:
        config.settimefrom(args.seconds)

    metrics = CyaniteMetrics(config)

    for metric in args.metric:
        metrics.prune(metric)

    if len(args.metric) == 0:
        for metric in fileinput.input(args.file):
            metrics.prune(metric.rstrip('\n'))



import yaml
import time

class Config():
    """
    Load the cyanite configuration.
    """

    def __init__(self, config_file):
        self.config_file = config_file
        self.verboseflag = False
        try:
            with open(config_file, 'r') as f:
                self.config = yaml.load(f)
        except IOError:
            self.config = dict()
        self.timewindow = 86400 * 3

    def cluster(self):
        casshosts = []
        if 'store' in self.config:
            if 'cluster' in self.config['store']:
                if isinstance(self.config['store']['cluster'], basestring):
                    casshosts.append(self.config['store']['cluster'])
                    return casshosts
                for host in self.config['store']['cluster']:
                    casshosts.append(host)
                return casshosts
        return ['localhost']

    def clusteruser(self):
        if 'store' in self.config:
            if 'username' in self.config['store']:
                return self.config['store']['username']
        return None

    def clusterpass(self):
        if 'store' in self.config:
            if 'password' in self.config['store']:
                return self.config['store']['password']
        return None

    def keyspace(self):
        if 'store' in self.config:
            if 'keyspace' in self.config['store']:
                return self.config['store']['keyspace']
        return 'metric'

    def httphost(self):
        return 'localhost'

    def httpport(self):
        if 'http' in self.config:
            if 'port' in self.config['http']:
                return self.config['http']['port']
        return '8080'

    def rollups(self):
        rollups = list()
        if 'carbon' in self.config:
            if 'rollups' in self.config['carbon']:
                for r in self.config['carbon']['rollups']:
                    rollups.append((r['period'], r['rollup']))
        return rollups

    def timefrom(self):
        now = int(time.time())
        return now - self.timewindow

    def settimefrom(self, timewindow):
        self.timewindow = int(timewindow)

    def verbose(self):
        return self.verboseflag

    def setverbose(self):
        self.verboseflag = True

    # True when a REST api index is defined
    def espathindex(self):
        if 'index' in self.config:
            if 'use' in self.config['index']:
                if self.config['index']['use'] == "io.cyanite.es_path/es-rest":
                    return True
        return False

    def esurl(self):
        if 'index' in self.config:
            if 'url' in self.config['index']:
                return self.config['index']['url']
        return 'http://localhost:9200'

    def esindex(self):
        if 'index' in self.config:
            if 'index' in self.config['index']:
                return self.config['index']['index']
        return 'cyanite_paths'



import time
import json
import urllib2
import sys
from .Config import Config

class CyanitePaths():
    """
    Connect to Cyanite and read metric paths
    """

    def __init__(self, config):
        self.config = config
        self.url = "http://%s:%s/paths" % (
                self.config.httphost(),
                self.config.httpport())
        self.esurl = "%s/%s" % (
                self.config.esurl(),
                self.config.esindex())
        self.cyanite = None

    def get(self, path):
        ret = list()
        if self.config.verbose():
            sys.stderr.write("path get %s\n" % path)
            sys.stderr.flush()
        url = "%s?query=%s" % (self.url, path)
        response = urllib2.urlopen(url)
        data = json.loads(response.read())
        if not data:
            return list()
        for item in data:
            metric = dict()
            if 'path' in item:
                metric['path'] = item['path']
                if 'leaf' in item:
                    metric['leaf'] = item['leaf']
                if 'tenant' in item and len(item['tenant']) > 0:
                    metric['tenant'] = item['tenant']
                ret.append(metric)
        return ret

    def printpaths(self, path):
        paths = self.get(path)
        for path in paths:
            if path['leaf']:
                sys.stdout.write("%s\n" % path['path'])
                sys.stdout.flush()
            else:
                self.printpaths("%s.*" % path['path'])

    # delete a path from elasticsearch
    def delete(self, path):
        ret = list()
        if self.config.verbose():
            sys.stderr.write("path delete %s\n" % path)
            sys.stderr.flush()
        url = "%s/path/%s" % (self.esurl, path)

        opener = urllib2.build_opener(urllib2.HTTPHandler)
        req = urllib2.Request(url)
        req.get_method = lambda: 'DELETE'
        try:
            response = urllib2.urlopen(req)
        except urllib2.HTTPError, err:
            if err.code == 404:
                sys.stderr.write("path delete %s does not exist\n" % path)
                sys.stderr.flush()
                return False
            else:
                raise
        data = json.loads(response.read())
        if not data:
            return False
        if 'found' in data and data['found']:
            return True
        return False



import time
import json
import urllib2
import sys
from .Config import Config
from .CyaniteCassandra import CyaniteCassandra

class CyaniteMetrics():
    """
    Connect to Cyanite and read metrics
    """

    def __init__(self, config):
        self.config = config
        self.url = "http://%s:%s/metrics" % (
                self.config.httphost(),
                self.config.httpport())
        self.cyanite = None
        self.paths = None
        self.maxrollup = 0
        for (rollup, interval) in self.config.rollups():
            if rollup * interval > self.maxrollup:
                self.maxrollup = rollup * interval - interval

    def get(self, path, timefrom=None, timeto=None):
        if not timefrom:
            timefrom = self.config.timefrom()
        if not timeto:
            timeto = int(time.time())
        if self.config.verbose():
            sys.stderr.write("metric get %s from %d to %d\n" % (
                path, timefrom, timeto))
            sys.stderr.flush()
        url = "%s?path=%s&from=%d&to=%d" % (self.url, path, timefrom, timeto)
        response = urllib2.urlopen(url)
        data = json.loads(response.read())
        return data

    def prune(self, path, timefrom=None):
        # open Cassandra connection
        if not self.cyanite:
            self.cyanite = CyaniteCassandra(self.config)

        if not timefrom:
            timefrom = self.config.timefrom()
        data = self.get(path, timefrom=timefrom)
        if not data:
            # no data, but prune the path
            sys.stdout.write("%s\n" % path)
            sys.stdout.flush()
            return False
        if 'series' in data and len(data['series']) == 0:
            sys.stdout.write("%s\n" % path)
            sys.stdout.flush()
        return True



import argparse

def common_parser(description='untitled'):
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument(
        '--config-file',
        default='/etc/cyanite.yaml',
        help='Config file to use')

    parser.add_argument(
        '-f', '--file',
        default='-',
        help='Input file')

    parser.add_argument(
        '-k', '--keyspace',
        default='metric',
        help='Keyspace name')

    parser.add_argument(
        '-H', '--cyanite-host',
        default='localhost',
        help='Cyanite hostname')

    parser.add_argument(
        '-p', '--http-port',
        default='8080',
        help='Cyanite http port')

    parser.add_argument(
        '-C', '--cassandra-host',
        default='localhost',
        help='Cassandra hostname')

    parser.add_argument(
        '-v', '--verbose',
        default=False,
        action='store_true',
        help='verbose')

    parser.add_argument(
        '--es-url',
        default='http://localhost:9200',
        help='Elasticsearch http port')

    parser.add_argument(
        '--es-index',
        default='cyanite_paths',
        help='Elasticsearch index')

    return parser

import signal
import sys
def cyanite_signal_handler(signal, frame):
    sys.exit(1)
def catch_sigint():
    signal.signal(signal.SIGINT, cyanite_signal_handler)



import os
import re
from cyanite_utils import __version__
from setuptools import setup, find_packages

setup(
    name='cyanite-utils',
    version=__version__,
    author="Chris Maxwell",
    author_email="chris@wrathofchris.com",
    description="Cyanite Utils",
    url = "https://github.com/WrathOfChris/cyanite-utils",
    download_url = 'https://github.com/WrathOfChris/cyanite-utils/tarball/0.0.10',
    license="Apache",
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'PyYAML',
        'cassandra-driver'
    ],
    entry_points={
        "console_scripts": [
            "cyanite-list = cyanite_utils.cli:cyanite_list",
            "cyanite-prune = cyanite_utils.cli:cyanite_prune",
            "cyanite-delete = cyanite_utils.cli:cyanite_delete"
        ]
    }
)
